/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.api;

import java.util.HashMap;
import java.util.List;

import org.drools.runtime.rule.FactHandle;

/**
 * A managed set of Drools working memories
 *
 * @author Julien Nicoulaud
 */
public interface IDroolsWorkingMemoryPool {

    // -------------------------------------------------------------------------
    // Set management
    // -------------------------------------------------------------------------

    /**
     * Add a WorkingMemory to the set, identified by a name.
     *
     * @param workingMemoryName the unique name
     * @param workingMemory the WorkingMemory
     * @throws Exception if the pool already contains a working memory
     * registered with the same name, the name is empty or the working memory
     * object is null.
     */
    public void add(String workingMemoryName,
                    IDroolsWorkingMemory workingMemory) throws Exception;

    /**
     * Remove a working memory from the set
     * Make sure to stop it before.
     *
     * @param workingMemoryName the targeted WorkingMemory
     * @return the WorkingMemory
     * @throws Exception if there is no working memory referenced
     * at that name.
     */
    public IDroolsWorkingMemory remove(String workingMemoryName) throws Exception;

    /**
     * Get a WorkingMemory in the set
     *
     * @param workingMemoryName the targeted WorkingMemory
     * @return the WorkingMemory or null if it does not exist
     */
    public IDroolsWorkingMemory get(String workingMemoryName);


    // -------------------------------------------------------------------------
    // Working memories batch control
    // -------------------------------------------------------------------------

    /**
     * Start all managed working memories
     *
     * @throws Exception if a WorkingMemory fails to start
     */
    public void startAll() throws Exception;

    /**
     * Stop all managed working memories
     */
    public void stopAll();

    /**
     * Insert a fact in all managed working memories
     * @param fact the fact object to insert
     * @return a map of <working memory name, FactHandle>
     */
    public HashMap<String, FactHandle> insertAll(Object fact);


    // -------------------------------------------------------------------------
    // Helpers
    // -------------------------------------------------------------------------

    /**
     * Get a WorkingMemory in the set, create it if does
     * not exist.
     * The potentially created WorkingMemory uses a default
     * ClassLoader.
     *
     * @param workingMemoryName the name of the new WorkingMemory
     * @throws Exception if a working memory could not be created.
     */
    public IDroolsWorkingMemory getOrCreate(String workingMemoryName) throws Exception;


    // -------------------------------------------------------------------------
    // Informations & statistics getters
    // -------------------------------------------------------------------------

    /**
     * Get a list of the managed working memories names.
     *
     * @return a list of the memories names as strings
     */
    public List<String> getWorkingMemoriesNames();

}
