/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.cluster.cmi;

import java.util.HashSet;
import java.util.Set;

import org.ow2.cmi.controller.server.ProtocolNotFoundException;
import org.ow2.cmi.controller.server.ServerClusterViewManager;
import org.ow2.cmi.reference.ServerId;
import org.ow2.cmi.reference.ServerNotFoundException;
import org.ow2.jasmine.rules.cluster.api.ServerState;

/**
 * Provide some action to reconfigure CMI.
 * @author Loris Bouzonnet
 */
public class CMIAction {

    /**
     * The cluster view manager.
     */
    private ServerClusterViewManager clusterViewManager;

    /**
     * Singleton.
     */
    private static CMIAction cmiAction = null;

    /**
     * No public constructor.
     */
    private CMIAction() {
        // Do nothing
    }

    /**
     * @return the only instance of CMIAction.
     */
    public static CMIAction getCMIAction() {
        if(cmiAction == null) {
            cmiAction = new CMIAction();
        }
        return cmiAction;
    }

    /**
     * Blacklist the CMI server.
     */
    public void blacklistMe(final String serverName) {
        if(clusterViewManager == null) {
            return;
        }
        for(ServerId serverId : getServerRefs(serverName)) {
            clusterViewManager.addServerToBlackList(serverId);
        }
    }

    /**
     * Unblacklist the CMI server.
     */
    public void unblacklistMe(final String serverName) {
        if(clusterViewManager == null) {
            return;
        }
        for(ServerId serverId : getServerRefs(serverName)) {
            clusterViewManager.removeServerFromBlackList(serverId);
        }
    }

    /**
     * Return state of the CMI server or null if it cannot be retrieved.
     * @return state of the CMI server or null if it cannot be retrieved
     */
    public ServerState getServerState(final String serverName) {
        if(clusterViewManager == null) {
            return null;
        }
        Set<ServerId> serverIds = getServerRefs(serverName);
        if(serverIds.isEmpty()) {
            return null;
        }
        boolean blacklisted = false;
        int loadFactor = Integer.MIN_VALUE;
        // Find the worst case: blacklisted and highest load factor
        try {
            for(ServerId serverId : serverIds) {
                blacklisted = blacklisted || clusterViewManager.isServerBlackListed(serverId);
                int lf = clusterViewManager.getLoadFactor(serverId);
                if(lf > loadFactor) {
                    loadFactor = lf;
                }
            }
        } catch (ServerNotFoundException e) {
            e.printStackTrace();
            return null;
        }
        return new ServerState(serverName, blacklisted, loadFactor);
    }


    /**
     * Set load factor of the CMI server
     * @param loadFactor load factor
     */
    public void setLoadFactor(final String serverName, final int loadFactor) {
        if(clusterViewManager == null) {
            return;
        }
        for(ServerId serverId : getServerRefs(serverName)) {
            clusterViewManager.setLoadFactor(serverId, loadFactor);
        }
    }

    /**
     * @param serverName server name
     * @return
     */
    private Set<ServerId> getServerRefs(final String serverName) {
        Set<ServerId> serverRefs = new HashSet<ServerId>();
        if(serverName == null) {
            for(String protocol : clusterViewManager.getProtocols()) {
                try {
                    serverRefs.add(clusterViewManager.getRefOnLocalRegistry(protocol));
                } catch (ProtocolNotFoundException e) {
                    e.printStackTrace();
                }
            }
        } else {
            serverRefs = clusterViewManager.getRefsOnRegistries(serverName);
        }
        return serverRefs;
    }

    /**
     * Delete the singleton.
     */
    public void destroy() {
        clusterViewManager = null;
        cmiAction = null;
    }

    /**
     * Set the cluster view manager
     * @param clusterViewManager the cluster view manager
     */
    public void setClusterViewManager(
            final ServerClusterViewManager clusterViewManager) {
        this.clusterViewManager = clusterViewManager;
    }

}
